# -*- coding: utf-8 -*-

#    Written by Martin v. Lwis <loewis@informatik.hu-berlin.de>
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys, os, getopt, struct, array

class msgfmt :

	MESSAGES = {}

	def usage(self, code, msg=''):
		print >> sys.stderr, __doc__
		if msg: print >> sys.stderr, msg

		sys.exit(code)

	def add(self, id, str, fuzzy):
		"Add a non-fuzzy translation to the dictionary."
		if not fuzzy and str:
			self.MESSAGES[id] = str

	def generate(self):
		"Return the generated output."
		keys = self.MESSAGES.keys()
		# the keys are sorted in the .mo file
		keys.sort()
		offsets = []
		ids = strs = ''
		for id in keys:
			# For each string, we need size and file offset.  Each string is NUL
			# terminated; the NUL does not count into the size.
			offsets.append((len(ids), len(id), len(strs), len(self.MESSAGES[id])))
			ids += id + '\0'
			strs += self.MESSAGES[id] + '\0'
		output = ''
		# The header is 7 32-bit unsigned integers.  We don't use hash tables, so
		# the keys start right after the index tables.
		# translated string.
		keystart = 7*4+16*len(keys)
		# and the values start after the keys
		valuestart = keystart + len(ids)
		koffsets = []
		voffsets = []
		# The string table first has the list of keys, then the list of values.
		# Each entry has first the size of the string, then the file offset.
		for o1, l1, o2, l2 in offsets:
			koffsets += [l1, o1+keystart]
			voffsets += [l2, o2+valuestart]
		offsets = koffsets + voffsets
		output = struct.pack("Iiiiiii",
									0x950412deL,	  # Magic
									0,				     # Version
									len(keys),		  # # of entries
									7*4,			     # start of key index
									7*4+len(keys)*8, # start of value index
									0, 0)			     # size and offset of hash table
		output += array.array("i", offsets).tostring()
		output += ids
		output += strs
		return output

	def make(self, filename):
		ID = 1
		STR = 2

		# Compute .mo name from .po name and arguments
		if filename.endswith('.po'):
			infile = filename
		else:
			infile = filename + '.po'

		outfile = os.path.splitext(infile)[0] + '.mo'

		try:
			lines = open(infile).readlines()
		except IOError, msg:
			print >> sys.stderr, msg
			sys.exit(1)

		section = None
		fuzzy = 0

		# Parse the catalog
		lno = 0
		for l in lines:
			lno += 1
			# If we get a comment line after a msgstr, this is a new entry
			if l[0] == '#' and section == STR:
					self.add(msgid, msgstr, fuzzy)
					section = None
					fuzzy = 0
			# Record a fuzzy mark
			if l[:2] == '#,' and l.find('fuzzy'):
					fuzzy = 1
			# Skip comments
			if l[0] == '#':
					continue
			# Now we are in a msgid section, output previous section
			if l.startswith('msgid'):
					if section == STR:
						self.add(msgid, msgstr, fuzzy)
					section = ID
					l = l[5:]
					msgid = msgstr = ''
			# Now we are in a msgstr section
			elif l.startswith('msgstr'):
					section = STR
					l = l[6:]
			# Skip empty lines
			l = l.strip()
			if not l:
					continue
			# XXX: Does this always follow Python escape semantics?
			l = eval(l)
			if section == ID:
					msgid += l
			elif section == STR:
					msgstr += l
			else:
					print >> sys.stderr, 'Syntax error on %s:%d' % (infile, lno), \
							'before:'
					print >> sys.stderr, l
					sys.exit(1)
		# Add last entry
		if section == STR:
			self.add(msgid, msgstr, fuzzy)

		# Compute output
		output = self.generate()

		try:
			open(outfile,"wb").write(output)
		except IOError,msg:
			print >> sys.stderr, msg

